@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2016 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2017 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -230,6 +230,14 @@ namespace KeePass.App.Configuration
 			}
 		}
 
+		private int m_iExpirySoonDays = 7;
+		[DefaultValue(7)]
+		public int ExpirySoonDays
+		{
+			get { return m_iExpirySoonDays; }
+			set { m_iExpirySoonDays = value; }
+		}
+
 		public string GetWorkingDirectory(string strContext)
 		{
 			// strContext may be null
