@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2016 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2017 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -268,6 +268,14 @@ namespace KeePass.App.Configuration
 			set { m_bDisableSaveIfNotModified = value; }
 		}
 
+		private bool m_bHideCloseDbTb = true;
+		[DefaultValue(true)]
+		public bool HideCloseDatabaseButton
+		{
+			get { return m_bHideCloseDbTb; }
+			set { m_bHideCloseDbTb = value; }
+		}
+
 		private AceToolBar m_tb = new AceToolBar();
 		public AceToolBar ToolBar
 		{
@@ -341,6 +349,14 @@ namespace KeePass.App.Configuration
 			set { m_bAlternatingBgColor = value; }
 		}
 
+		private int m_argbAltBgColor = 0;
+		[DefaultValue(0)]
+		public int EntryListAlternatingBgColor
+		{
+			get { return m_argbAltBgColor; }
+			set { m_argbAltBgColor = value; }
+		}
+
 		private bool m_bResolveFieldRefs = false;
 		[DefaultValue(false)]
 		public bool EntryListShowDerefData
