@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2016 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2017 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -310,6 +310,14 @@ namespace KeePass.App.Configuration
 		//	set { m_bEditCancelConfirmation = value; }
 		// }
 
+		private bool m_bAdvATCmds = false;
+		[DefaultValue(false)]
+		public bool ShowAdvAutoTypeCommands
+		{
+			get { return m_bAdvATCmds; }
+			set { m_bAdvATCmds = value; }
+		}
+
 		private bool m_bSecDeskSound = true;
 		[DefaultValue(true)]
 		public bool SecureDesktopPlaySound
