@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2016 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2017 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -19,21 +19,22 @@
 
 using System;
 using System.Collections.Generic;
-using System.Text;
-using System.Runtime.InteropServices;
-using System.Globalization;
 using System.Diagnostics;
+using System.Globalization;
+using System.Runtime.InteropServices;
+using System.Text;
 
 using KeePass.Resources;
 
 using KeePassLib;
-using KeePassLib.Security;
 using KeePassLib.Collections;
+using KeePassLib.Cryptography.KeyDerivation;
 using KeePassLib.Delegates;
 using KeePassLib.Interfaces;
 using KeePassLib.Keys;
-using KeePassLib.Utility;
 using KeePassLib.Resources;
+using KeePassLib.Security;
+using KeePassLib.Utility;
 
 namespace KeePass.DataExchange
 {
@@ -139,7 +140,9 @@ namespace KeePass.DataExchange
 					throw new Exception(KLRes.FileLoadFailed);
 
 				// Copy properties
-				m_pwDatabase.KeyEncryptionRounds = mgr.KeyTransformationRounds;
+				m_pwDatabase.KdfParameters = (new AesKdf()).GetDefaultParameters();
+				m_pwDatabase.KdfParameters.SetUInt64(AesKdf.ParamRounds,
+					mgr.KeyTransformationRounds);
 
 				// Read groups and entries
 				Dictionary<UInt32, PwGroup> dictGroups = ReadGroups(mgr);
@@ -283,9 +286,17 @@ namespace KeePass.DataExchange
 				}
 
 				// Set properties
-				if(m_pwDatabase.KeyEncryptionRounds >= (ulong)UInt32.MaxValue)
-					mgr.KeyTransformationRounds = 0xFFFFFFFEU;
-				else mgr.KeyTransformationRounds = (uint)m_pwDatabase.KeyEncryptionRounds;
+				AesKdf kdf = new AesKdf();
+				if(!kdf.Uuid.Equals(m_pwDatabase.KdfParameters.KdfUuid))
+					mgr.KeyTransformationRounds = (uint)PwDefs.DefaultKeyEncryptionRounds;
+				else
+				{
+					ulong uRounds = m_pwDatabase.KdfParameters.GetUInt64(
+						AesKdf.ParamRounds, PwDefs.DefaultKeyEncryptionRounds);
+					uRounds = Math.Min(uRounds, 0xFFFFFFFEUL);
+
+					mgr.KeyTransformationRounds = (uint)uRounds;
+				}
 
 				PwGroup pgRoot = (pgDataSource ?? m_pwDatabase.RootGroup);
 
