@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2016 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2017 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,10 +20,11 @@
 using System;
 using System.Collections.Generic;
 using System.ComponentModel;
+using System.Diagnostics;
 using System.Drawing;
 using System.Text;
+using System.Threading;
 using System.Windows.Forms;
-using System.Diagnostics;
 
 using KeePass.App;
 using KeePass.App.Configuration;
@@ -33,6 +34,7 @@ using KeePass.Resources;
 using KeePassLib;
 using KeePassLib.Delegates;
 using KeePassLib.Cryptography.Cipher;
+using KeePassLib.Cryptography.KeyDerivation;
 using KeePassLib.Keys;
 using KeePassLib.Security;
 using KeePassLib.Utility;
@@ -48,6 +50,7 @@ namespace KeePass.Forms
 
 		private ContextMenu m_ctxColor = null;
 		private List<ColorMenuItem> m_vColorItems = new List<ColorMenuItem>();
+		private Image m_imgColor = null;
 
 		private string m_strAutoCreateNew = "(" + KPRes.AutoCreateNew + ")";
 		private Dictionary<int, PwUuid> m_dictRecycleBinGroups = new Dictionary<int, PwUuid>();
@@ -55,6 +58,12 @@ namespace KeePass.Forms
 		private Dictionary<int, PwUuid> m_dictEntryTemplateGroups = new Dictionary<int, PwUuid>();
 
 		private bool m_bInitializing = true;
+		private volatile Thread m_thKdf = null;
+
+		public PwDatabase DatabaseEx
+		{
+			get { return m_pwDatabase; }
+		}
 
 		public DatabaseSettingsForm()
 		{
@@ -90,7 +99,8 @@ namespace KeePass.Forms
 			FontUtil.AssignDefaultBold(m_rbNone);
 			FontUtil.AssignDefaultBold(m_rbGZip);
 
-			m_ttRect.SetToolTip(m_lnkCompute1SecDelay, KPRes.TransformationRounds1SecHint);
+			UIUtil.ConfigureToolTip(m_ttRect);
+			m_ttRect.SetToolTip(m_btnKdf1Sec, KPRes.KdfParams1Sec);
 
 			m_tbDbName.PromptText = KPRes.DatabaseNamePrompt;
 			m_tbDbDesc.PromptText = KPRes.DatabaseDescPrompt;
@@ -104,8 +114,8 @@ namespace KeePass.Forms
 
 			m_clr = m_pwDatabase.Color;
 			if(m_clr != Color.Empty)
-				UIUtil.SetButtonImage(m_btnColor, PwEntryForm.CreateColorButtonImage(
-					m_btnColor, m_clr), false);
+				UIUtil.OverwriteButtonImage(m_btnColor, ref m_imgColor,
+					UIUtil.CreateColorBitmap24(m_btnColor, m_clr));
 			m_cbColor.Checked = (m_clr != Color.Empty);
 
 			for(int inx = 0; inx < CipherPool.GlobalPool.EngineCount; ++inx)
@@ -117,9 +127,29 @@ namespace KeePass.Forms
 				m_cmbEncAlgo.SelectedIndex = ((nIndex >= 0) ? nIndex : 0);
 			}
 
-			m_numEncRounds.Minimum = ulong.MinValue;
-			m_numEncRounds.Maximum = ulong.MaxValue;
-			m_numEncRounds.Value = m_pwDatabase.KeyEncryptionRounds;
+			Debug.Assert(m_cmbKdf.Items.Count == 0);
+			foreach(KdfEngine kdf in KdfPool.Engines)
+			{
+				m_cmbKdf.Items.Add(kdf.Name);
+			}
+
+			m_numKdfIt.Minimum = ulong.MinValue;
+			m_numKdfIt.Maximum = ulong.MaxValue;
+
+			m_numKdfMem.Minimum = ulong.MinValue;
+			m_numKdfMem.Maximum = ulong.MaxValue;
+
+			Debug.Assert(m_cmbKdfMem.Items.Count == 0);
+			Debug.Assert(!m_cmbKdfMem.Sorted);
+			m_cmbKdfMem.Items.Add("B");
+			m_cmbKdfMem.Items.Add("KB");
+			m_cmbKdfMem.Items.Add("MB");
+			m_cmbKdfMem.Items.Add("GB");
+
+			m_numKdfPar.Minimum = uint.MinValue;
+			m_numKdfPar.Maximum = uint.MaxValue;
+
+			SetKdfParameters(m_pwDatabase.KdfParameters);
 
 			// m_lbMemProt.Items.Add(KPRes.Title, m_pwDatabase.MemoryProtection.ProtectTitle);
 			// m_lbMemProt.Items.Add(KPRes.UserName, m_pwDatabase.MemoryProtection.ProtectUserName);
@@ -193,6 +223,8 @@ namespace KeePass.Forms
 			bool bChangeForce = (m_pwDatabase.MasterKeyChangeForce >= 0);
 			m_numKeyForceDays.Value = (bChangeForce ? m_pwDatabase.MasterKeyChangeForce : 365);
 			m_cbKeyForce.Checked = bChangeForce;
+
+			m_cbKeyForceOnce.Checked = m_pwDatabase.MasterKeyChangeForceOnce;
 		}
 
 		private void EnableControlsEx()
@@ -201,6 +233,31 @@ namespace KeePass.Forms
 
 			m_btnColor.Enabled = m_cbColor.Checked;
 
+			KdfEngine kdf = GetKdf();
+			if(kdf != null)
+			{
+				if(kdf is AesKdf)
+				{
+					m_lblKdfMem.Visible = false;
+					m_numKdfMem.Visible = false;
+					m_cmbKdfMem.Visible = false;
+
+					m_lblKdfPar.Visible = false;
+					m_numKdfPar.Visible = false;
+				}
+				else if(kdf is Argon2Kdf)
+				{
+					m_lblKdfMem.Visible = true;
+					m_numKdfMem.Visible = true;
+					m_cmbKdfMem.Visible = true;
+
+					m_lblKdfPar.Visible = true;
+					m_numKdfPar.Visible = true;
+				}
+				// else { Debug.Assert(false); } // Plugins may provide controls
+			}
+			else { Debug.Assert(false); }
+
 			m_numHistoryMaxItems.Enabled = m_cbHistoryMaxItems.Checked;
 			m_numHistoryMaxSize.Enabled = m_cbHistoryMaxSize.Checked;
 
@@ -210,14 +267,28 @@ namespace KeePass.Forms
 			m_numKeyForceDays.Enabled = (bEnableDays && m_cbKeyForce.Checked);
 			m_cbKeyRec.Enabled = bEnableDays;
 			m_cbKeyForce.Enabled = bEnableDays;
+			m_cbKeyForceOnce.Enabled = bEnableDays;
+
+			bool bKdfTh = (m_thKdf != null);
+			m_tabMain.Enabled = !bKdfTh;
+
+			m_pbKdf.Visible = bKdfTh;
+
+			m_btnHelp.Enabled = !bKdfTh;
+			m_btnCancelOp.Enabled = bKdfTh;
+			m_btnCancelOp.Visible = bKdfTh;
+			m_btnOK.Enabled = !bKdfTh;
+			m_btnCancel.Enabled = !bKdfTh;
 		}
 
 		private void OnBtnOK(object sender, EventArgs e)
 		{
+			m_pwDatabase.SettingsChanged = DateTime.UtcNow;
+
 			if(!m_tbDbName.Text.Equals(m_pwDatabase.Name))
 			{
 				m_pwDatabase.Name = m_tbDbName.Text;
-				m_pwDatabase.NameChanged = DateTime.Now;
+				m_pwDatabase.NameChanged = DateTime.UtcNow;
 			}
 
 			string strNew = m_tbDbDesc.Text;
@@ -226,13 +297,13 @@ namespace KeePass.Forms
 			if(!strNewFlt.Equals(strOrgFlt))
 			{
 				m_pwDatabase.Description = strNew;
-				m_pwDatabase.DescriptionChanged = DateTime.Now;
+				m_pwDatabase.DescriptionChanged = DateTime.UtcNow;
 			}
 
 			if(!m_tbDefaultUser.Text.Equals(m_pwDatabase.DefaultUserName))
 			{
 				m_pwDatabase.DefaultUserName = m_tbDefaultUser.Text;
-				m_pwDatabase.DefaultUserNameChanged = DateTime.Now;
+				m_pwDatabase.DefaultUserNameChanged = DateTime.UtcNow;
 			}
 
 			if(!m_cbColor.Checked) m_pwDatabase.Color = Color.Empty;
@@ -245,7 +316,10 @@ namespace KeePass.Forms
 			else
 				m_pwDatabase.DataCipherUuid = StandardAesEngine.AesUuid;
 
-			m_pwDatabase.KeyEncryptionRounds = (ulong)m_numEncRounds.Value;
+			// m_pwDatabase.KeyEncryptionRounds = (ulong)m_numKdfIt.Value;
+			KdfParameters pKdf = GetKdfParameters(true);
+			if(pKdf != null) m_pwDatabase.KdfParameters = pKdf;
+			// No assert, plugins may assign KDF parameters
 
 			if(m_rbNone.Checked) m_pwDatabase.Compression = PwCompressionAlgorithm.None;
 			else if(m_rbGZip.Checked) m_pwDatabase.Compression = PwCompressionAlgorithm.GZip;
@@ -267,7 +341,7 @@ namespace KeePass.Forms
 			if(m_cbRecycleBin.Checked != m_pwDatabase.RecycleBinEnabled)
 			{
 				m_pwDatabase.RecycleBinEnabled = m_cbRecycleBin.Checked;
-				m_pwDatabase.RecycleBinChanged = DateTime.Now;
+				m_pwDatabase.RecycleBinChanged = DateTime.UtcNow;
 			}
 			int iRecBinSel = m_cmbRecycleBin.SelectedIndex;
 			if(m_dictRecycleBinGroups.ContainsKey(iRecBinSel))
@@ -275,7 +349,7 @@ namespace KeePass.Forms
 				if(!m_dictRecycleBinGroups[iRecBinSel].Equals(m_pwDatabase.RecycleBinUuid))
 				{
 					m_pwDatabase.RecycleBinUuid = m_dictRecycleBinGroups[iRecBinSel];
-					m_pwDatabase.RecycleBinChanged = DateTime.Now;
+					m_pwDatabase.RecycleBinChanged = DateTime.UtcNow;
 				}
 			}
 			else
@@ -284,7 +358,7 @@ namespace KeePass.Forms
 				if(!PwUuid.Zero.Equals(m_pwDatabase.RecycleBinUuid))
 				{
 					m_pwDatabase.RecycleBinUuid = PwUuid.Zero;
-					m_pwDatabase.RecycleBinChanged = DateTime.Now;
+					m_pwDatabase.RecycleBinChanged = DateTime.UtcNow;
 				}
 			}
 
@@ -294,7 +368,7 @@ namespace KeePass.Forms
 				if(!m_dictEntryTemplateGroups[iTemplSel].Equals(m_pwDatabase.EntryTemplatesGroup))
 				{
 					m_pwDatabase.EntryTemplatesGroup = m_dictEntryTemplateGroups[iTemplSel];
-					m_pwDatabase.EntryTemplatesGroupChanged = DateTime.Now;
+					m_pwDatabase.EntryTemplatesGroupChanged = DateTime.UtcNow;
 				}
 			}
 			else
@@ -303,7 +377,7 @@ namespace KeePass.Forms
 				if(!PwUuid.Zero.Equals(m_pwDatabase.EntryTemplatesGroup))
 				{
 					m_pwDatabase.EntryTemplatesGroup = PwUuid.Zero;
-					m_pwDatabase.EntryTemplatesGroupChanged = DateTime.Now;
+					m_pwDatabase.EntryTemplatesGroupChanged = DateTime.UtcNow;
 				}
 			}
 
@@ -320,6 +394,8 @@ namespace KeePass.Forms
 
 			if(!m_cbKeyForce.Checked) m_pwDatabase.MasterKeyChangeForce = -1;
 			else m_pwDatabase.MasterKeyChangeForce = (long)m_numKeyForceDays.Value;
+
+			m_pwDatabase.MasterKeyChangeForceOnce = m_cbKeyForceOnce.Checked;
 		}
 
 		// private bool UpdateMemoryProtection(int nIndex, bool bOldSetting,
@@ -360,19 +436,28 @@ namespace KeePass.Forms
 			AppHelp.ShowHelp(AppDefs.HelpTopics.DatabaseSettings, strSubTopic);
 		}
 
-		private void OnLinkClicked1SecondDelayRounds(object sender, LinkLabelLinkClickedEventArgs e)
+		private bool AbortKdfThread()
 		{
-			if(e.Button == MouseButtons.Left)
-				m_numEncRounds.Value = CompositeKey.TransformKeyBenchmark(1000, 3001);
+			if(m_thKdf == null) return false;
+
+			try { m_thKdf.Abort(); }
+			catch(Exception) { Debug.Assert(false); }
+			m_thKdf = null;
+
+			return true;
 		}
 
 		private void OnFormClosed(object sender, FormClosedEventArgs e)
 		{
+			if(AbortKdfThread()) { Debug.Assert(false); }
+
 			GlobalWindowManager.RemoveWindow(this);
 
 			foreach(ColorMenuItem mi in m_vColorItems)
 				mi.Click -= this.HandleColorButtonClicked;
 			m_vColorItems.Clear();
+
+			UIUtil.DisposeButtonImage(m_btnColor, ref m_imgColor);
 		}
 
 		private void OnKeyRecCheckedChanged(object sender, EventArgs e)
@@ -407,8 +492,8 @@ namespace KeePass.Forms
 			if(mi == null) { Debug.Assert(false); return; }
 
 			m_clr = mi.Color;
-			UIUtil.SetButtonImage(m_btnColor, PwEntryForm.CreateColorButtonImage(
-				m_btnColor, m_clr), false);
+			UIUtil.OverwriteButtonImage(m_btnColor, ref m_imgColor,
+				UIUtil.CreateColorBitmap24(m_btnColor, m_clr));
 		}
 
 		private void OnBtnColor(object sender, EventArgs e)
@@ -419,8 +504,8 @@ namespace KeePass.Forms
 			//	float h, s, v;
 			//	UIUtil.ColorToHsv(clr.Value, out h, out s, out v);
 			//	m_clr = UIUtil.ColorFromHsv(h, 1.0f, 1.0f);
-			//	UIUtil.SetButtonImage(m_btnColor, PwEntryForm.CreateColorButtonImage(
-			//		m_btnColor, m_clr), false);
+			//	UIUtil.OverwriteButtonImage(m_btnColor, ref m_imgColor,
+			//		UIUtil.CreateColorBitmap24(m_btnColor, m_clr));
 			// }
 
 			if(m_ctxColor == null)
@@ -488,5 +573,341 @@ namespace KeePass.Forms
 		{
 			EnableControlsEx();
 		}
+
+		private KdfEngine GetKdf()
+		{
+			return KdfPool.Get(m_cmbKdf.Text);
+		}
+
+		private KdfParameters GetKdfParameters(bool bShowAdjustments)
+		{
+			KdfEngine kdf = GetKdf();
+			if(kdf == null) { Debug.Assert(false); return null; }
+
+			string strAdj = string.Empty;
+
+			KdfParameters pKdf = kdf.GetDefaultParameters();
+			if(kdf is AesKdf)
+				pKdf.SetUInt64(AesKdf.ParamRounds, (ulong)m_numKdfIt.Value);
+			else if(kdf is Argon2Kdf)
+			{
+				ulong uIt = (ulong)m_numKdfIt.Value;
+				AdjustKdfParam<ulong>(ref uIt, ">=", Argon2Kdf.MinIterations,
+					KPRes.Iterations, ref strAdj);
+				AdjustKdfParam<ulong>(ref uIt, "<=", Argon2Kdf.MaxIterations,
+					KPRes.Iterations, ref strAdj);
+				pKdf.SetUInt64(Argon2Kdf.ParamIterations, uIt);
+
+				// Adjust parallelism first, as memory depends on it
+				uint uPar = (uint)m_numKdfPar.Value;
+				AdjustKdfParam<uint>(ref uPar, ">=", Argon2Kdf.MinParallelism,
+					KPRes.Parallelism, ref strAdj);
+				uint uParMax = Argon2Kdf.MaxParallelism;
+				int cp = Environment.ProcessorCount;
+				if((cp > 0) && (cp <= (int.MaxValue / 2)))
+					uParMax = Math.Min(uParMax, (uint)(cp * 2));
+				AdjustKdfParam<uint>(ref uPar, "<=", uParMax,
+					KPRes.Parallelism, ref strAdj);
+				pKdf.SetUInt32(Argon2Kdf.ParamParallelism, uPar);
+
+				ulong uMem = (ulong)m_numKdfMem.Value;
+				int iMemUnit = m_cmbKdfMem.SelectedIndex;
+				while(iMemUnit > 0)
+				{
+					if(uMem > (ulong.MaxValue / 1024UL))
+					{
+						uMem = ulong.MaxValue;
+						break;
+					}
+
+					uMem *= 1024UL;
+					--iMemUnit;
+				}
+
+				// 8*p blocks = 1024*8*p bytes minimum memory, see spec
+				Debug.Assert(Argon2Kdf.MinMemory == (1024UL * 8UL));
+				ulong uMemMin = Argon2Kdf.MinMemory * uPar;
+				AdjustKdfParam<ulong>(ref uMem, ">=", uMemMin,
+					KPRes.Memory, ref strAdj);
+				AdjustKdfParam<ulong>(ref uMem, "<=", Argon2Kdf.MaxMemory,
+					KPRes.Memory, ref strAdj);
+				pKdf.SetUInt64(Argon2Kdf.ParamMemory, uMem);
+			}
+			else return null; // Plugins may handle it
+
+			if(bShowAdjustments && (strAdj.Length > 0))
+			{
+				strAdj = KPRes.KdfAdjust + MessageService.NewParagraph + strAdj;
+				MessageService.ShowInfo(strAdj);
+			}
+
+			return pKdf;
+		}
+
+		private static void AdjustKdfParam<T>(ref T tValue, string strReq,
+			T tCmp, string strName, ref string strAdj)
+			where T : struct, IComparable<T>
+		{
+			if(strReq == ">=")
+			{
+				if(tValue.CompareTo(tCmp) >= 0) return;
+			}
+			else if(strReq == "<=")
+			{
+				if(tValue.CompareTo(tCmp) <= 0) return;
+			}
+			else { Debug.Assert(false); return; }
+
+			if(strAdj.Length > 0)
+				strAdj += MessageService.NewLine;
+			strAdj += "* " + strName + ": " + tValue.ToString() +
+				" -> " + tCmp.ToString() + ".";
+
+			tValue = tCmp;
+		}
+
+		private void SetKdfParameters(KdfParameters p)
+		{
+			if(p == null) { Debug.Assert(false); return; }
+
+			KdfEngine kdf = KdfPool.Get(p.KdfUuid);
+			if(kdf == null) { Debug.Assert(false); return; }
+
+			for(int i = 0; i < m_cmbKdf.Items.Count; ++i)
+			{
+				string strKdf = (m_cmbKdf.Items[i] as string);
+				if(string.IsNullOrEmpty(strKdf)) { Debug.Assert(false); continue; }
+
+				if(strKdf.Equals(kdf.Name, StrUtil.CaseIgnoreCmp))
+				{
+					bool bPrevInit = m_bInitializing;
+					m_bInitializing = true; // Prevent selection handler
+
+					m_cmbKdf.SelectedIndex = i;
+
+					m_bInitializing = bPrevInit;
+					break;
+				}
+			}
+
+			if(kdf is AesKdf)
+			{
+				ulong uIt = p.GetUInt64(AesKdf.ParamRounds,
+					PwDefs.DefaultKeyEncryptionRounds);
+				SetKdfParameters(uIt, 1024, 2);
+			}
+			else if(kdf is Argon2Kdf)
+			{
+				ulong uIt = p.GetUInt64(Argon2Kdf.ParamIterations,
+					Argon2Kdf.DefaultIterations);
+				ulong uMem = p.GetUInt64(Argon2Kdf.ParamMemory,
+					Argon2Kdf.DefaultMemory);
+				uint uPar = p.GetUInt32(Argon2Kdf.ParamParallelism,
+					Argon2Kdf.DefaultParallelism);
+				SetKdfParameters(uIt, uMem, uPar);
+			}
+			// else { Debug.Assert(false); } // Plugins may provide other KDFs
+		}
+
+		private void SetKdfParameters(ulong uIt, ulong uMem, uint uPar)
+		{
+			m_numKdfIt.Value = uIt;
+
+			int nUnits = m_cmbKdfMem.Items.Count;
+			if((nUnits == 0) || (uMem == 0))
+			{
+				m_numKdfMem.Value = uMem;
+				if(nUnits > 0) m_cmbKdfMem.SelectedIndex = 0;
+				else { Debug.Assert(false); }
+				return;
+			}
+
+			ulong uMemWrtUnit = uMem;
+			int iUnit = 0;
+			while(iUnit < (nUnits - 1))
+			{
+				if((uMemWrtUnit % 1024UL) != 0UL) break;
+
+				uMemWrtUnit /= 1024UL;
+				++iUnit;
+			}
+
+			m_numKdfMem.Value = uMemWrtUnit;
+			m_cmbKdfMem.SelectedIndex = iUnit;
+
+			m_numKdfPar.Value = uPar;
+		}
+
+		private void OnKdfSelectedIndexChanged(object sender, EventArgs e)
+		{
+			if(m_bInitializing) return;
+
+			KdfEngine kdf = GetKdf();
+			if(kdf == null) { Debug.Assert(false); return; }
+
+			SetKdfParameters(kdf.GetDefaultParameters());
+			EnableControlsEx();
+		}
+
+		private void OnBtnKdf1Sec(object sender, EventArgs e)
+		{
+			KdfEngine kdf = GetKdf();
+			if(kdf == null) { Debug.Assert(false); return; }
+
+			if(!(kdf is AesKdf) && !(kdf is Argon2Kdf)) return; // No assert, plugins
+			if(m_thKdf != null) { Debug.Assert(false); return; }
+
+			try
+			{
+				m_thKdf = new Thread(new ParameterizedThreadStart(this.Kdf1SecTh));
+				EnableControlsEx(); // Disable controls (m_thKdf is not null)
+
+				m_thKdf.Start(kdf);
+			}
+			catch(Exception)
+			{
+				Debug.Assert(false);
+				m_thKdf = null;
+				EnableControlsEx();
+			}
+		}
+
+		private void Kdf1SecTh(object o)
+		{
+			KdfParameters p = null;
+			string strMsg = null;
+
+			try
+			{
+				KdfEngine kdf = (o as KdfEngine);
+				if(kdf != null) p = kdf.GetBestParameters(1000);
+				else { Debug.Assert(false); }
+			}
+			catch(ThreadAbortException)
+			{
+				try { Thread.ResetAbort(); }
+				catch(Exception) { Debug.Assert(false); }
+				return;
+			}
+			catch(Exception ex)
+			{
+				if((ex != null) && !string.IsNullOrEmpty(ex.Message))
+					strMsg = ex.Message;
+			}
+			finally { m_thKdf = null; } // Before continuation, to enable controls
+
+			try { m_btnOK.Invoke(new KdfpDelegate(this.Kdf1SecPost), p, strMsg); }
+			catch(Exception) { Debug.Assert(false); }
+		}
+
+		private delegate void KdfpDelegate(KdfParameters p, string strMsg);
+
+		private void Kdf1SecPost(KdfParameters p, string strMsg)
+		{
+			try
+			{
+				Debug.Assert(!m_btnOK.InvokeRequired);
+
+				if(!string.IsNullOrEmpty(strMsg))
+					MessageService.ShowInfo(strMsg);
+				else SetKdfParameters(p);
+
+				EnableControlsEx();
+			}
+			catch(Exception) { Debug.Assert(false); }
+		}
+
+		private void OnBtnKdfTest(object sender, EventArgs e)
+		{
+			KdfParameters p = GetKdfParameters(true);
+			if(p == null) return; // No assert, plugins
+
+			if(m_thKdf != null) { Debug.Assert(false); return; }
+
+			try
+			{
+				SetKdfParameters(p); // Show auto-adjusted parameters
+
+				m_thKdf = new Thread(new ParameterizedThreadStart(this.KdfTestTh));
+				EnableControlsEx(); // Disable controls (m_thKdf is not null)
+
+				m_thKdf.Start(p);
+			}
+			catch(Exception)
+			{
+				Debug.Assert(false);
+				m_thKdf = null;
+				EnableControlsEx();
+			}
+		}
+
+		private void KdfTestTh(object o)
+		{
+			string strMsg = KPRes.UnknownError;
+
+			try
+			{
+				KdfParameters p = (o as KdfParameters);
+				if(p == null) { Debug.Assert(false); return; }
+
+				KdfEngine kdf = KdfPool.Get(p.KdfUuid);
+				if(kdf == null) { Debug.Assert(false); return; }
+
+				byte[] pbMsg = new byte[32];
+				Program.GlobalRandom.NextBytes(pbMsg);
+
+				kdf.Randomize(p);
+
+				Stopwatch sw = Stopwatch.StartNew();
+				kdf.Transform(pbMsg, p);
+				sw.Stop();
+
+				long lMS = sw.ElapsedMilliseconds;
+				lMS = Math.Max(lMS, 1L);
+				double dS = (double)lMS / 1000.0;
+
+				strMsg = KPRes.TestSuccess + MessageService.NewParagraph +
+					KPRes.TransformTime.Replace(@"{PARAM}", dS.ToString());
+			}
+			catch(ThreadAbortException)
+			{
+				try { Thread.ResetAbort(); }
+				catch(Exception) { Debug.Assert(false); }
+				return;
+			}
+			catch(Exception ex)
+			{
+				Debug.Assert(false);
+				if((ex != null) && !string.IsNullOrEmpty(ex.Message))
+					strMsg = ex.Message;
+			}
+			finally { m_thKdf = null; } // Before continuation, to enable controls
+
+			try { m_btnOK.Invoke(new KdfsDelegate(this.KdfTestPost), strMsg); }
+			catch(Exception) { Debug.Assert(false); }
+		}
+
+		private delegate void KdfsDelegate(string strMsg);
+
+		private void KdfTestPost(string strMsg)
+		{
+			try
+			{
+				Debug.Assert(!m_btnOK.InvokeRequired);
+
+				if(!string.IsNullOrEmpty(strMsg))
+					MessageService.ShowInfo(strMsg);
+				else { Debug.Assert(false); }
+
+				EnableControlsEx();
+			}
+			catch(Exception) { Debug.Assert(false); }
+		}
+
+		private void OnBtnCancelOp(object sender, EventArgs e)
+		{
+			AbortKdfThread();
+			EnableControlsEx();
+		}
 	}
-}
\ No newline at end of file
+}
