@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2016 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2017 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -56,6 +56,8 @@ namespace KeePass.Forms
 		private CheckedLVItemDXList m_cdxAdvanced = null;
 
 		private Dictionary<int, string> m_dTsrUuids = new Dictionary<int, string>();
+		private int m_argbAltItemBg = 0;
+		private Image m_imgAltItemBg = null;
 
 		private HotKeyControlEx m_hkGlobalAutoType = null;
 		private HotKeyControlEx m_hkSelectedAutoType = null;
@@ -408,6 +410,10 @@ namespace KeePass.Forms
 				lvg, KPRes.ShowFullPathInTitleBar);
 			m_cdxGuiOptions.CreateItem(Program.Config.MainWindow, "DisableSaveIfNotModified",
 				lvg, KPRes.DisableSaveIfNotModified);
+			m_cdxGuiOptions.CreateItem(Program.Config.MainWindow, "HideCloseDatabaseButton",
+				lvg, KPRes.HideCloseDatabaseTb);
+			m_cdxGuiOptions.CreateItem(Program.Config.UI, "ShowAdvAutoTypeCommands",
+				lvg, KPRes.ShowAdvAutoTypeCommands);
 
 			lvg = new ListViewGroup(KPRes.EntryList);
 			m_lvGuiOptions.Groups.Add(lvg);
@@ -486,6 +492,10 @@ namespace KeePass.Forms
 				m_numMruCount.Enabled = false;
 			}
 
+			m_argbAltItemBg = Program.Config.MainWindow.EntryListAlternatingBgColor;
+			m_cbCustomAltColor.Checked = (m_argbAltItemBg != 0);
+			UpdateButtonImages();
+
 			if(AppConfigEx.IsOptionEnforced(Program.Config.UI, "StandardFont"))
 				m_btnSelFont.Enabled = false;
 			if(AppConfigEx.IsOptionEnforced(Program.Config.UI, "PasswordFont") ||
@@ -706,6 +716,9 @@ namespace KeePass.Forms
 			Program.Config.Application.MostRecentlyUsed.MaxItemCount =
 				(uint)m_numMruCount.Value;
 
+			Program.Config.MainWindow.EntryListAlternatingBgColor =
+				(m_cbCustomAltColor.Checked ? m_argbAltItemBg : 0);
+
 			ChangeHotKey(ref m_kPrevATHKKey, m_hkGlobalAutoType,
 				AppDefs.GlobalHotKeyId.AutoType);
 			ChangeHotKey(ref m_kPrevATSHKKey, m_hkSelectedAutoType,
@@ -731,6 +744,8 @@ namespace KeePass.Forms
 
 			m_tabMain.ImageList = null; // Detach event handlers
 
+			UIUtil.DisposeButtonImage(m_btnCustomAltColor, ref m_imgAltItemBg);
+
 			m_cdxSecurityOptions.Release();
 			m_cdxPolicy.Release();
 			m_cdxGuiOptions.Release();
@@ -783,6 +798,8 @@ namespace KeePass.Forms
 			m_numClipClearTime.Enabled = (m_cbClipClearTime.Checked &&
 				m_cbClipClearTime.Enabled);
 
+			m_btnCustomAltColor.Enabled = m_cbCustomAltColor.Checked;
+
 			m_bBlockUIUpdate = false;
 		}
 
@@ -940,5 +957,34 @@ namespace KeePass.Forms
 			ProxyForm dlg = new ProxyForm();
 			UIUtil.ShowDialogAndDestroy(dlg);
 		}
+
+		private void UpdateButtonImages()
+		{
+			if(m_argbAltItemBg != 0)
+			{
+				Color clr = Color.FromArgb(m_argbAltItemBg);
+				Image imgNew = UIUtil.CreateColorBitmap24(m_btnCustomAltColor, clr);
+				UIUtil.OverwriteButtonImage(m_btnCustomAltColor, ref m_imgAltItemBg,
+					imgNew);
+			}
+		}
+
+		private void OnCustomAltColorCheckedChanged(object sender, EventArgs e)
+		{
+			UpdateUIState();
+		}
+
+		private void OnBtnCustomAltColor(object sender, EventArgs e)
+		{
+			Color clrCur = UIUtil.GetAlternateColor(m_lvGuiOptions.BackColor);
+			if(m_argbAltItemBg != 0) clrCur = Color.FromArgb(m_argbAltItemBg);
+
+			Color? clr = UIUtil.ShowColorDialog(clrCur);
+			if(clr.HasValue)
+			{
+				m_argbAltItemBg = clr.Value.ToArgb();
+				UpdateButtonImages();
+			}
+		}
 	}
 }
