@@ -1,6 +1,6 @@
 ﻿/*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2016 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2017 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -482,8 +482,8 @@ namespace KeePass
 
 			AutoType.InitStatic();
 
-			UserActivityNotifyFilter nfActivity = new UserActivityNotifyFilter();
-			Application.AddMessageFilter(nfActivity);
+			CustomMessageFilterEx cmfx = new CustomMessageFilterEx();
+			Application.AddMessageFilter(cmfx);
 
 #if DEBUG
 			if(m_cmdLineArgs[AppDefs.CommandLineOptions.DebugThrowException] != null)
@@ -509,7 +509,7 @@ namespace KeePass
 			}
 #endif
 
-			Application.RemoveMessageFilter(nfActivity);
+			Application.RemoveMessageFilter(cmfx);
 
 			Debug.Assert(GlobalWindowManager.WindowCount == 0);
 			Debug.Assert(MessageService.CurrentMessageCount == 0);
@@ -540,6 +540,7 @@ namespace KeePass
 			m_rndGlobal = new Random(nRandomSeed);
 
 			InitEnvSecurity();
+			MonoWorkarounds.Initialize();
 
 			// try { NativeMethods.SetProcessDPIAware(); }
 			// catch(Exception) { }
@@ -594,6 +595,7 @@ namespace KeePass
 			}
 
 			EnableThemingInScope.StaticDispose();
+			MonoWorkarounds.Terminate();
 		}
 
 		private static void MainCleanUp()
