@@ -1,6 +1,6 @@
 ﻿/*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2016 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2017 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -145,7 +145,7 @@ namespace KeePass.UI
 
 			try
 			{
-				if(this.DoubleBuffered)
+				if(this.DoubleBuffered && !NativeLib.IsUnix())
 				{
 					IntPtr p = new IntPtr((int)NativeMethods.TVS_EX_DOUBLEBUFFER);
 					NativeMethods.SendMessage(this.Handle,
@@ -153,7 +153,7 @@ namespace KeePass.UI
 				}
 				else { Debug.Assert(!WinUtil.IsAtLeastWindowsVista); }
 			}
-			catch(Exception) { Debug.Assert(NativeLib.IsUnix()); }
+			catch(Exception) { Debug.Assert(false); }
 
 			// Display tooltips for a longer time;
 			// https://sourceforge.net/p/keepass/feature-requests/2038/
