@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2016 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2017 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -479,8 +479,10 @@ namespace KeePass.Util
 			byte[] pbHash;
 			try
 			{
-				SHA256Managed sha256 = new SHA256Managed();
-				pbHash = sha256.ComputeHash(sIn);
+				using(SHA256Managed sha256 = new SHA256Managed())
+				{
+					pbHash = sha256.ComputeHash(sIn);
+				}
 			}
 			catch(Exception) { Debug.Assert(false); sIn.Close(); return null; }
 
